\name{mcmc.risksets2}
\alias{mcmc.risksets2}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ This function performs clustering of the risk sets}
\description{
 The algorithm identifies the components in the risk sets that are frequently sampled and adds more components and clusters risk sets together with these components. The algorith keeps adding components until the joint posterior for the cluster of risk sets get below a specified threshold
}
\usage{
mcmc.risksets2(mcmc, thres = 0.1, file = paste(mcmc$name,
                 "risksets.txt", sep = ""), change = NULL, change_from
                 = NULL, change_to = NULL, max_risksets = 50, rm.empty
                 = T, rm.empty.riskset0 = F, rm.no.parameters = T,
                 min.nr.in.riskset = 1, return = F, env.integer = T,
                 burnin = 0, log = F, latex = F, change_all = NULL,
                 strict = F)

}
\arguments{
  \item{mcmc}{ an mcmc object }
  \item{thres}{The threshold for the minimum posterior for each risk sets. A low threshold will give many clusters. }
  \item{file}{A filename (for saving the results) }
  \item{change}{a vektor of component numbers (integers) that manually changes the state of a component (see details) }
  \item{change_from}{a vektor of component states that will be change (see details) }
  \item{change_to}{ a vektor of component states that the states in change\_from should be changed to  (see details)}
  \item{max_risksets}{Integer. The maximum number of clusters }
  \item{rm.empty}{Boolean. If TRUE the sampled risk sets that does not partition the individuals are removed}
  \item{rm.empty.riskset0}{ Boolean. If TRUE all the sampled risk sets in an iteration where all the individuals are placed in a risk set are removed }
  \item{rm.no.parameters}{NA }
  \item{min.nr.in.riskset}{Integer. The minimum number of individuals in a risk group. If rm.empty is TRUE risk sets that give rise to risk groups with less than this number of individuals will be removed. This can be handly if there are som extreme outliers in the data that are of little interest }
  \item{return}{Boolean. It TRUE the result is returned }
  \item{env.integer}{NA }
  \item{burnin}{Integer. The burnin of the chain }
  \item{log}{Boolean. If the trait was transformed using the logarithm
the results can be transformed back }'
\item{latex}{print latex file}
\item{change_all}{change all}
\item{strict}{strict}
}
\details{

Manually merging component states:
Sometimes different states of the components will partition the data in the same way and these can be manually merged. For example if there are only a few homozygotes for the minor allele then the state that allows only heterozygotes and the state that allows both heterozygotes and homozygotes (for the minor allele) will partition the data in the same way. There these can manually merged. The states for the SNP components are given as 1: WT (homozygote for the major allele), 2: HE (heterozygote), 3: HO (homozygote for the minor allele), 5: WT or HE, 6: HE or HO, 7: WT or HO. If there are only a few homozygotes for the minor allele I suggest merging state 2 and 6. e.g. for SNP 5 and 6: change=c(5,6),change\_from=c(2,2),change\_to=c(6,6)



}
\value{
  \item{short.table }{the clusters of risk sets}
  \item{nr}{the 90 \% credibility interval (CI)for the number of individuals}
  \item{adj_mean}{The mean and 90\% CI for the means of the risk sets. If any adjustment factors were included in the MCMC run the mean is adjusted}
  \item{post}{The number of sampled risk sets with these components (e.g. alleles or environmental factors) divided by the number of iterations (this in not a real posterior probability)}
  \item{post.only}{The number of sampled risk sets with these components and no other components divided by the number of iterations (this in not a real posterior probability)}
  \item{post.strict}{The posterior probability including these components in a risk set}
  \item{post.only.strict}{The posterior probability including these components and no other components in a risk set}
  \item{WT}{Wildtype or the genotype included in the mcmc run as 1}
  \item{HE}{Heterozygote or the genotype included in the mcmc run as 2}
  \item{HO}{Homozygote or the genotype included in the mcmc run as 3}
  \item{HE/HO}{Heterozygote or Homozygote}
  \item{misc.post}{The number of sampled risk sets non in any cluster (misc risk sets) divided by the number of iterations }
  \item{misc.mean}{the mean for the misc risk sets}
  \item{misc.mean05}{the 90 \% lower CI for the mean of the misc risk sets}
  \item{misc.mean95}{the 90 \% upper CI for the mean of the misc risk sets}
  \item{misc.ind05 }{the 90 \% lower CI for the number of individuals in the misc risk sets}
  \item{misc.ind95 }{the 90 \% upper CI for the number of individuals in the misc risk sets}
  \item{non.mean}{The mean for the individuals not placed in a risk group}
  \item{non.mean05}{the 90 \% lower CI for the mean of the non risk sets}
  \item{non.mean85}{the 90 \% upper CI for the mean of the non risk sets}
  \item{non.ind05 }{the 90 \% lower CI for the number of individuals in the non risk sets}
  \item{non.ind95 }{the 90 \% upper CI for the number of individuals in the non risk sets}
  \item{mean.sd}{the mean of the standard deviation}
}
\references{ ~put references to the literature/web site here ~ }
\author{anders albrechtsen }
\note{ ~~further notes~~ 

 ~Make other sections like Warning with \section{Warning }{....} ~
}
\seealso{ ~~objects to See Also as \code{\link{help}}, ~~~ }

\keyword{ cluster }